{$I uev3defines.inc}
unit pseudolresources;
interface 
uses sysutils;

type
 TPseudoLResource = class
  public
    Name: AnsiString;
    ValueType: AnsiString;
    Value: AnsiString;
  end;

 { TPseudoLResourceList }

 TPseudoLResourceList=class
  private
    FList: array of TPseudoLResource;  // main list with all resource pointers
    function GetItems(Index: integer): TPseudoLResource;
  public
    constructor Create;
    destructor Destroy;  override;
    procedure Add(const Name, ValueType, Value: AnsiString);
    procedure Add(const Name, ValueType: AnsiString; const Values: array of string);
    function Find(const Name: AnsiString): TPseudoLResource; overload;
    function Find(const Name, ValueType: AnsiString): TPseudoLResource; overload;
    function Count: integer;
    property Items[Index: integer]: TPseudoLResource read GetItems;
    function Last: TPseudoLResource;
    function First: TPseudoLResource;
    procedure Delete(const Name, ValueType: AnsiString);
    procedure Delete(const Name: AnsiString);
    procedure DeleteLast;
  end;

var LazarusResources,
    LRS:TPseudoLResourceList; // ресурс и псевдоним
    CLR:TPseudoLResource; // свободная переменная 

implementation


{ TPseudoLResourceList }

function TPseudoLResourceList.Last: TPseudoLResource;
begin
 if length(flist)=0 then exit(nil) else exit(flist[length(flist)-1]);
end;

function TPseudoLResourceList.First: TPseudoLResource;
begin
 if length(flist)=0 then exit(nil) else exit(flist[0]);
end;

procedure TPseudoLResourceList.DeleteLast;
begin
 if assigned(last) then begin
    last.free;
    flist[length(flist)-1]:=nil;
 end;
end;


procedure TPseudoLResourceList.Delete(const Name: AnsiString);
 var R:TPseudoLResource; i:integer;
begin
  for i:=0 to length(flist)-1 do begin
   R:=flist[i];
   if assigned(r) and (name=r.name) then begin
    r.free;
    flist[i]:=nil;
   end;
  end;
end;

procedure TPseudoLResourceList.Delete(const Name, ValueType: AnsiString);
 var R:TPseudoLResource; i:integer;
begin
  for i:=0 to length(flist)-1 do begin
   R:=flist[i];
   if assigned(r) and (name=r.name) and (valueType=r.valuetype) then begin
    r.free;
    flist[i]:=nil;
   end;
  end;
end;

function TPseudoLResourceList.GetItems(Index: integer): TPseudoLResource;
begin
  result:=FList[index];
end;

constructor TPseudoLResourceList.Create;
begin
 inherited.create;
 FList:=nil;
end;

destructor TPseudoLResourceList.Destroy;
 var res:TPseudoLResource;
begin
  for res in FList do res.free;
  flist:=nil;
  inherited Destroy;
end;

procedure TPseudoLResourceList.Add(const Name, ValueType, Value: AnsiString);
 var res:TPseudoLResource;
begin
  res:=TPseudoLResource.Create;
  res.Name:=Name;
  res.ValueType:=uppercase(ValueType);
  res.Value:=Value;
  setlength(FList,length(FList)+1);
  FList[length(FList)-1]:=res;
end;

procedure TPseudoLResourceList.Add(const Name, ValueType: AnsiString;
  const Values: array of string);
  var str,sumstr:Ansistring;
begin
  sumstr:='';
  for str in Values do sumstr+=str;
  add(Name,ValueType,sumstr);
end;

function TPseudoLResourceList.Find(const Name: AnsiString): TPseudoLResource;
 var res:TPseudoLResource;
begin
 for res in FList do
  if assigned(res) and (res.Name=Name) then exit(res);
 result:=nil;
end;

function TPseudoLResourceList.Find(const Name, ValueType: AnsiString
  ): TPseudoLResource;
 var res:TPseudoLResource;
begin
 for res in flist do
  if assigned(res)and(res.Name=Name)and(res.ValueType=uppercase(ValueType)) then exit(res);
 result:=nil;
end;

function TPseudoLResourceList.Count: integer;
begin
 result:=length(FList)
end;

initialization
 LazarusResources:=TPseudoLResourceList.Create;
 LRS:=LazarusResources;
finalization
 LRS.free;
end.

