{
    Файл является частью библиотеки RubiRobot

    © 2017-2020 Слинкин Д.А.

    Обработка аудио-системы контроллера Lego EV3

    Подробная информация о лицензировании находится в файле README,
    входящем в состав библиотеки.

    ДАННОЕ ПРОГРАММНОЕ ОБЕСПЕЧЕНИЕ ПРЕДОСТАВЛЯЕТСЯ «КАК ЕСТЬ», БЕЗ
    КАКИХ-ЛИБО ГАРАНТИЙ, ЯВНО ВЫРАЖЕННЫХ ИЛИ ПОДРАЗУМЕВАЕМЫХ,
    ВКЛЮЧАЯ ГАРАНТИИ ТОВАРНОЙ ПРИГОДНОСТИ, СООТВЕТСТВИЯ ПО ЕГО
    КОНКРЕТНОМУ НАЗНАЧЕНИЮ И ОТСУТСТВИЯ НАРУШЕНИЙ, НО НЕ ОГРАНИЧИВАЯСЬ 
    ИМИ. НИ В КАКОМ СЛУЧАЕ АВТОРЫ ИЛИ ПРАВООБЛАДАТЕЛИ НЕ НЕСУТ
    ОТВЕТСТВЕННОСТИ ПО КАКИМ-ЛИБО ИСКАМ, ЗА УЩЕРБ ИЛИ ПО ИНЫМ ТРЕБОВАНИЯМ,
    В ТОМ ЧИСЛЕ, ПРИ ДЕЙСТВИИ КОНТРАКТА, ДЕЛИКТЕ ИЛИ ИНОЙ СИТУАЦИИ,
    ВОЗНИКШИМ ИЗ-ЗА ИСПОЛЬЗОВАНИЯ ПРОГРАММНОГО ОБЕСПЕЧЕНИЯ ИЛИ ИНЫХ
    ДЕЙСТВИЙ С ПРОГРАММНЫМ ОБЕСПЕЧЕНИЕМ.

 **********************************************************************}
{$I uev3defines.inc}

unit rubiroSound;
interface
 uses unix,uev3;
type
  { TEv3Sound }
  TEv3Sound=class(TEv3)
    private
      beepfi:cint;
      beeprec:TInputEvent;
      beepProcess:cint; // мьютекс 9
      playProcess:cint; // мьютекс 8
      speakInProcess,speakOutProcess:cint; // мьютекс 7
      fstopBeforeStart:boolean;
      fspeakSpeed:integer;
      function getStopBeforeStart: boolean;
      procedure setStopBeforeStart(AValue: boolean);
    protected
      procedure realBeep(beepcmd:string; args:array of ansistring);
    public
      constructor create;
      destructor destroy;override;
      // Устанавливает уровень громкости (от 0 до 100%)
      function volume(vol:variant):TEv3Sound;
      {
        Подает звуковой сигнал заданной частоты напрямую на устройство.
        Работает очень быстро, без вызовов внешних программ.
        Игнорирует звуковую очередь.
        Для остановки сигнала - передать 0, либо вызвать beep
      }
      function beepDirect(tone:variant):TEv3Sound;
      {
       Подает звуковой сигнал заданной частоты, заданной длительности, с заданной паузой
       по окончании. Длительность по умолчанию = 200мс, пауза - 0мс
      }
      function beep(tone:variant):TEv3Sound; // -f tone -l duration -D delay -n
      function beep(tone:variant;duration:variant):TEv3Sound; // -f tone -l duration -D delay -n
      function beep(tone:variant;duration:variant;delay:variant):TEv3Sound; // -f tone -l duration -D delay -n
      //             450           200              0
      {
       Подает набор звуковых сигналов заданной частоты, заданной длительности, с заданной задержкой
       по окончании. Информация о сигналах передается в массив тройками частота-длительность-пауза.
      }
      function beep(tdws:array of const):TEv3Sound; // тройки частота-длительность-пауза
      // Озвучивает файл в WAV-формате, имя которого передается в функцию
      function play(wavfile:variant):TEv3Sound;
      {
       Озвучивает переданный текст синтезатором речи,
       скорость по умолчанию = 160 слов в минуту
      }
      function speak(txt:variant):TEv3Sound; //speed=160
      function speak(txt:variant; speed:variant):TEv3Sound; //speed=160
      function speak(txts:array of const):TEv3Sound;
      function speak(txts:array of const; speed:variant):TEv3Sound;
      // устанавливает кол-во слов в минуту для синтезатора речи
      function setSpeakSpeed(speed:variant):TEv3Sound;
      // Ожидает завершения любых запущенных звуковых процессов
      function wait():TEv3Sound; // all
      {
        Ожидает завершения запущенных звуковых процессов
        Параметр what определяет тип процессов:
        'all' - все звуковые процессы
        'beep' - звуковой сигнал
        'play' - озвучивание WAV-файла
        'speak' - синтезатор речи
      }
      function wait(what:variant):TEv3Sound; // all, beep, play или speak
      // Прерывает все запущенные звуковые процессы
      function stop():TEv3Sound; // all, beep, play и speak
      // Прерывает запущенные звуковые процессы, параметр what - см. wait()
      function stop(what:variant):TEv3Sound; // all, beep, beepdirect, play или speak
      {
       устанавливает режим ожидания завершения звукового процесса перед
       стартом нового звукового процесса такого-же типа. Используется
       по умолчанию
      }
      function setWaitBeforeStart():TEv3Sound; // default
      {
       устанавливает режим прерывания звукового процесса перед
       стартом нового звукового процесса такого-же типа.
      }
      function setStopBeforeStart():TEv3Sound;
      property stopBeforeStart:boolean read getStopBeforeStart write SetstopBeforeStart;

  end;

var ev3Sound:TEv3Sound=nil;


implementation
uses baseunix,sysutils,variants,uev3func;

__WARNGLOBALOFF

{ TEv3Sound }

procedure TEv3Sound.setStopBeforeStart(AValue: boolean);
begin
  if FstopBeforeStart=AValue then Exit;
  FstopBeforeStart:=AValue;
end;

procedure TEv3Sound.realBeep(beepcmd: string; args: array of ansistring);
begin
  _lock(9);
  try
    if beepProcess<>0 then begin
      if stopBeforeStart then stop('beep')
      else wait('beep');
    end;
    if length(args)=0 then beepProcess:=startProcess(beepcmd)
    else beepProcess:=startProcess(beepcmd,args);
  finally
    _unlock(9);
  end;
end;

function TEv3Sound.getStopBeforeStart: boolean;
begin
  result:=fStopBeforeStart;
end;

constructor TEv3Sound.create;
begin
  inherited create;
  beepfi:=fpopen('/dev/input/by-path/platform-snd-legoev3-event',O_WRonly);
  if beepfi<0 then beepfi:=fpopen('/dev/input/by-path/platform-sound-event',O_WRonly);
  beeprec.type_event:=EV_SND;
  beeprec.event_code:=SND_TONE;
  beeprec.value:=0;

  beepProcess:=0;
  playProcess:=0;
  speakInProcess:=0;
  speakOutProcess:=0;
  stopBeforeStart:=false;
  fspeakSpeed:=160;
end;

destructor TEv3Sound.destroy;
begin
  stop('all');
  beepDirect(0);
  fpclose(beepfi);
  inherited destroy;
end;

function TEv3Sound.volume(vol: variant): TEv3Sound;
begin
 result:=self;
 fpsystem('amixer -q set Master '+varToStr(vol)+'%'+devnull);
 fpsystem('amixer -q set Playback '+varToStr(vol)+'%'+devnull);
end;

function TEv3Sound.beepDirect(tone: variant): TEv3Sound;
begin
  result:=self;
  _lock(6);
  beeprec.value:=tone;
  fpwrite(beepfi,beeprec,sizeof(beeprec));
  _unlock(6);
end;

function TEv3Sound.beep(tone: variant): TEv3Sound;
 var a:array of ansistring=nil;
begin
  result:=self;
  setlength(a,0);
  if isStr(tone) then begin
   realBeep('/usr/bin/beep '+toStr(tone),a);
  end
  else beep([tone,200,0]);
end;

function TEv3Sound.beep(tone: variant; duration: variant): TEv3Sound;
begin
   result:=self;
   beep([tone,duration,0]);
end;

function TEv3Sound.beep(tone: variant; duration: variant; delay: variant
  ): TEv3Sound;
begin
   result:=self;
   beep([tone,duration,delay])
end;

function TEv3Sound.beep(tdws: array of const): TEv3Sound;
   Var I : longint;
     //  s:string;
       a:array of ansistring;
 begin
     result:=self;
     if High(tdws)<0 then exit;
     a:=nil;
//     -f tone -l duration -D delay -n
     i:=0;
     while i<=High(tdws) do begin
       push(a,'-f');
       push(a,arg2string(tdws[i]));
       push(a,'-l');
       push(a,arg2string(tdws[i+1]));
       push(a,'-D');
       push(a,arg2string(tdws[i+2]));
       push(a,'-n');
       inc(i,3);
     end;
     pop(a);
     realBeep('/usr/bin/beep',a);
{
     _lock(9);
     try
       if beepProcess<>0 then begin
         if stopBeforeStart then stop('beep')
         else wait('beep');
       end;
       beepProcess:=startProcess('/usr/bin/beep',a);
     finally
       _unlock(9);
     end;
}

 end;

function TEv3Sound.play(wavfile: variant): TEv3Sound;
begin
    result:=self;
    _lock(8);
    try
     if playProcess<>0 then begin
      if stopBeforeStart then stop('play')
      else wait('play');
     end;
     playProcess:=startProcess('/usr/bin/aplay', ['-q','-N',vartostr(wavfile)]);
    finally
      _unlock(8);
    end;
end;

function TEv3Sound.speak(txt: variant): TEv3Sound;
begin
  result:=self;
  speak([txt],fspeakspeed);
end;

function TEv3Sound.speak(txt: variant; speed: variant): TEv3Sound;
begin
    result:=self;
    speak([txt],speed);
end;

function TEv3Sound.speak(txts: array of const): TEv3Sound;
begin
  result:=self;
  speak(txts,fspeakspeed);
end;

function TEv3Sound.speak(txts: array of const; speed: variant): TEv3Sound;
Var I : longint;
    s:string;
    a1:array of ansistring;
begin
  result:=self;
  s:='';
  if High(txts)<0 then exit;
  a1:=nil;
  push(a1,'-s');
  push(a1,varToStr(speed));
  push(a1,'-a');
  push(a1,'200');
  push(a1,'-v');
  push(a1,'ru');
  push(a1,'--stdout');
  for i:=0 to High(txts) do begin
    s:=s+arg2string(txts[i])+' ';
  end;
  push(a1,'"'+s+'"');
  _lock(7);
  try
   if speakInProcess<>0 then begin
    if stopBeforeStart then stop('speak')
    else wait('speak');
   end;
   StartPipedProcess('/usr/bin/espeak',a1,'/usr/bin/aplay',[],speakInProcess,speakOutProcess);
  finally
    _unlock(7);
  end;
end;

function TEv3Sound.wait: TEv3Sound;
begin
  result:=self;
  wait('all');
end;

function TEv3Sound.wait(what: variant): TEv3Sound;
begin
  result:=self;
  case vartostr(what) of
   'all': begin wait('beep'); wait('play'); wait('speak'); end;
   'speak':begin
     _lock(7);
     try
       if speakInprocess<>0 then begin
        fpWaitPid(speakInprocess,nil,0);
        fpWaitPid(speakOutprocess,nil,0);
        speakInprocess:=0;
        speakOutprocess:=0;
       end;
     finally
      _unlock(7);
     end;
   end;
   'play': begin
     _lock(8);
     try
       if playprocess<>0 then fpWaitPid(playprocess,nil,0);
       playprocess:=0;
     finally
      _unlock(8);
     end;
   end;
   'beep': begin
     _lock(9);
     try
       if beepprocess<>0 then fpWaitPid(beepprocess,nil,0);
       beepprocess:=0;
     finally
       _unlock(9);
     end;
   end;
  end;
end;

function TEv3Sound.stop: TEv3Sound;
begin
  result:=self;
  stop('all');
end;

function TEv3Sound.stop(what: variant): TEv3Sound;
begin
  result:=self;
  case vartostr(what) of
   'all': begin stop('beep'); stop('play'); stop('speak'); stop('beepdirect'); end;
   'speak': begin
     _lock(7);
     try
       if speakInprocess<>0 then begin
        try killProcess(speakInprocess); except end;
        try killProcess(speakOutprocess); except end;
        speakInprocess:=0;
        speakOutprocess:=0;
       end;
     finally
       _unlock(7);
     end;
   end;
   'play': begin
     _lock(8);
     try
      if playprocess<>0 then begin
       //writeln('pkill ',playprocess);
       try killProcess(playprocess); except end;
       //fpwaitpid(playprocess,nil,0);
      end;
      playprocess:=0;
     finally
      _unlock(8);
     end;
   end;
   'beep': begin
     _lock(9);
     try
       if beepprocess<>0 then begin
        try killProcess(beepprocess);except end;
        //fpwaitpid(beepprocess,nil,0);
       end;
       beepprocess:=0;
     finally
      _unlock(9);
     end;
   end;
   'beepdirect': beepdirect(0);
  end;
end;

function TEv3Sound.setWaitBeforeStart: TEv3Sound;
begin
  result:=self;
  stopBeforeStart:=false;
end;

function TEv3Sound.setStopBeforeStart: TEv3Sound;
begin
   result:=self;
   stopBeforeStart:=true;
end;

function TEv3Sound.setSpeakSpeed(speed: variant): TEv3Sound;
begin
  result:=self;
  fspeakSpeed:=speed;
end;


procedure init; begin ev3Sound:=TEv3Sound.create(); end;
procedure done; begin ev3Sound.free; end;

initialization
 PushInit(@init);
  PushDone(@Done);
end.

