{
    Файл является частью библиотеки RubiRobot

    © 2017-2020 Слинкин Д.А.

    Класс монохромного изображения с экспортом-импортом на fb-устройство
    Используется для поддержки дисплея контроллера Lego EV3

    Подробная информация о лицензировании находится в файле README,
    входящем в состав библиотеки.

    ДАННОЕ ПРОГРАММНОЕ ОБЕСПЕЧЕНИЕ ПРЕДОСТАВЛЯЕТСЯ «КАК ЕСТЬ», БЕЗ
    КАКИХ-ЛИБО ГАРАНТИЙ, ЯВНО ВЫРАЖЕННЫХ ИЛИ ПОДРАЗУМЕВАЕМЫХ,
    ВКЛЮЧАЯ ГАРАНТИИ ТОВАРНОЙ ПРИГОДНОСТИ, СООТВЕТСТВИЯ ПО ЕГО
    КОНКРЕТНОМУ НАЗНАЧЕНИЮ И ОТСУТСТВИЯ НАРУШЕНИЙ, НО НЕ ОГРАНИЧИВАЯСЬ 
    ИМИ. НИ В КАКОМ СЛУЧАЕ АВТОРЫ ИЛИ ПРАВООБЛАДАТЕЛИ НЕ НЕСУТ
    ОТВЕТСТВЕННОСТИ ПО КАКИМ-ЛИБО ИСКАМ, ЗА УЩЕРБ ИЛИ ПО ИНЫМ ТРЕБОВАНИЯМ,
    В ТОМ ЧИСЛЕ, ПРИ ДЕЙСТВИИ КОНТРАКТА, ДЕЛИКТЕ ИЛИ ИНОЙ СИТУАЦИИ,
    ВОЗНИКШИМ ИЗ-ЗА ИСПОЛЬЗОВАНИЯ ПРОГРАММНОГО ОБЕСПЕЧЕНИЯ ИЛИ ИНЫХ
    ДЕЙСТВИЙ С ПРОГРАММНЫМ ОБЕСПЕЧЕНИЕМ.

 **********************************************************************}
{$I uev3defines.inc}

unit uev3image;
interface
uses uev3,fpimage;

const
  ev3WhiteColor:TFPColor = (red:$ffff;green:$ffff;blue:$ffff;alpha:$ffff);
  ev3BlackColor:TFPColor = (red:0;green:0;blue:0;alpha:$ffff);
  ev3Width=178;
  ev3MaxWidth=192;
  ev3Height=128;
  ev3bpl=24;
  ev3Cbpl=712;
  { TEv3Image }
type
  TEv3fbmem=array[0..ev3Height-1,0..ev3bpl-1]of byte;
  TEv3Cfbmem=array[0..ev3Height-1,0..ev3Cbpl-1]of byte;

  TEv3Image=class(TFPCustomImage)
    private
      Finverse: boolean;
      procedure Setinverse(AValue: boolean);
    protected
      procedure SetUsePalette (Value:boolean);override;
    public
      constructor create();overload;
      procedure clear(); virtual; abstract;
      procedure exportFB(var fbmem); virtual; abstract;
      procedure importFB(const fbmem); virtual; abstract;
      property inverse:boolean read Finverse write Setinverse;
  end;

  { TEv3ImageBW }

  TEv3ImageBW=class(TEv3Image)
    private
      fMem:array[0..ev3Height-1,0..ev3MaxWidth-1]of byte;
    protected
      function GetInternalColor(x,y:integer):TFPColor;override;
      procedure SetInternalColor (x,y:integer; const Value:TFPColor);override;
      procedure SetInternalPixel (x,y:integer; Value:integer); override;
      function GetInternalPixel (x,y:integer) : integer; override;
    public
      procedure clear(); override;
      procedure exportFB(var fbmem);override;
      procedure importFB(const fbmem);override;
  end;

  { TEv3ImageXRGB }

  TXRGB=packed record b,g,r,x:byte; end;
  TEv3ImageXRGB=class(TEv3Image)
    private
      fMem:array[0..ev3Height-1,0..ev3Width-1]of TXRGB;
    protected
      function GetInternalColor(x,y:integer):TFPColor;override;
      procedure SetInternalColor (x,y:integer; const Value:TFPColor);override;
      procedure SetInternalPixel (x,y:integer; Value:integer); override;
      function GetInternalPixel (x,y:integer) : integer; override;
    public
      procedure clear(); override;
      procedure exportFB(var fbmem);override;
      procedure importFB(const fbmem);override;
  end;




implementation

var
 ev3BlackA:array[0..3]of byte=(255,255,255,255);
 ev3WhiteA:array[0..3]of byte=(0,0,0,0);

{ TEv3ImageXRGB }


function TEv3ImageXRGB.GetInternalColor(x, y: integer): TFPColor;
  var pixel:integer=0;
      xrgb:txrgb absolute pixel;
begin
  pixel:=GetInternalPixel(x, y);
  result.Alpha:=$ffff;
  if finverse then begin
    result.Red:=not (xrgb.r shl 8);
    result.Green:=not (xrgb.g shl 8);
    result.Blue:=not (xrgb.b shl 8);
  end
  else begin
   result.Red:= (xrgb.r shl 8);
   result.Green:= (xrgb.g shl 8);
   result.Blue:= (xrgb.b shl 8);
  end;
end;

procedure TEv3ImageXRGB.SetInternalColor(x, y: integer; const Value: TFPColor);
 var pixel:integer=0;
     xrgb:txrgb absolute pixel;
begin
  if finverse then begin
    xrgb.r:=not (Value.Red shr 8);
    xrgb.g:=not (Value.Green shr 8);
    xrgb.b:= not (Value.Blue shr 8);
  end
  else begin
   xrgb.r:=Value.Red shr 8;
   xrgb.g:=Value.Green shr 8;
   xrgb.b:=Value.Blue shr 8;
  end;
  SetInternalPixel(x, y, pixel);
  //  writeln(x,' ',y,' ',value.red,' ',value.green,' ',value.blue,' ',value.alpha);
//  if value=ev3whiteColor then SetInternalPixel(x, y, (ev3White xor ord(finverse)))
//  else SetInternalPixel(x, y, (ev3Black xor ord(finverse)))
end;


procedure TEv3ImageXRGB.SetInternalPixel(x, y: integer; Value: integer);
begin
  if (x<0)or(x>=ev3Width)or(y<0)or(y>=ev3Height) then exit;
  fMem[y,x]:=txrgb(value);
end;

function TEv3ImageXRGB.GetInternalPixel(x, y: integer): integer;
begin
  if (x<0)or(x>=ev3Width)or(y<0)or(y>=ev3Height) then begin result:=0; exit; end;
  result:=integer(fMem[y,x]);
end;

procedure TEv3ImageXRGB.clear();
begin
  if finverse then
   fillbyte(fmem,sizeof(fmem),0)
  else
   fillbyte(fmem,sizeof(fmem),255);
end;

procedure TEv3ImageXRGB.exportFB(var fbmem);
begin
   move(fmem,fbmem,sizeof(fmem))
end;

procedure TEv3ImageXRGB.importFB(const fbmem);
begin
  move(fbmem,fmem,sizeof(fmem))
end;


{ TEv3ImageBW }

function TEv3ImageBW.GetInternalColor(x, y: integer): TFPColor;
begin
  if GetInternalPixel(x, y)=(ev3Black xor ord(finverse)) then result:=ev3BlackColor
  else result:=ev3WhiteColor;
end;

procedure TEv3ImageBW.SetInternalColor(x, y: integer; const Value: TFPColor);
begin
//  writeln(x,' ',y,' ',value.red,' ',value.green,' ',value.blue,' ',value.alpha);
  if value=ev3whiteColor then SetInternalPixel(x, y, (ev3White xor ord(finverse)))
  else SetInternalPixel(x, y, (ev3Black xor ord(finverse)))
end;


procedure TEv3ImageBW.SetInternalPixel(x, y: integer; Value: integer);
begin
  if (x<0)or(x>=ev3MaxWidth)or(y<0)or(y>=ev3Height) then exit;
  fMem[y,x]:=value;
end;

function TEv3ImageBW.GetInternalPixel(x, y: integer): integer;
begin
  if (x<0)or(x>=ev3MaxWidth)or(y<0)or(y>=ev3Height) then begin result:=0; exit; end;
  result:=fMem[y,x];
end;

procedure TEv3ImageBW.clear();
begin
  fillbyte(fmem,sizeof(fmem),0);
end;

procedure TEv3ImageBW.exportFB(var fbmem);
 var m:Tev3fbmem absolute fbmem;
     p:pbyte; i,j:integer;
begin
  p:=@fmem;
  for i:=0 to ev3Height-1 do begin
   for j:=0 to ev3bpl-1 do begin
      m[i,j]:=p[0] or (p[1] shl 1) or (p[2] shl 2) or (p[3] shl 3) or (p[4] shl 4)
                   or (p[5] shl 5) or (p[6] shl 6) or (p[7] shl 7);
      inc(p,8);
   end;
  end;
end;

procedure TEv3ImageBW.importFB(const fbmem);
 var m:Tev3fbmem absolute fbmem;
     p:pbyte; b:byte; i,j:integer;
begin
  p:=@fmem;
  for i:=0 to ev3Height-1 do begin
   for j:=0 to ev3bpl-1 do begin
	b:=m[i,j];
	p[0]:=b and 1;
	p[1]:=(b shr 1) and 1;
	p[2]:=(b shr 2) and 1;
	p[3]:=(b shr 3) and 1;
	p[4]:=(b shr 4) and 1;
	p[5]:=(b shr 5) and 1;
	p[6]:=(b shr 6) and 1;
	p[7]:=(b shr 7) and 1;
      inc(p,8);
   end;
  end;
end;

{ TEv3Image }

__WARNOFFUnused
procedure TEv3Image.SetUsePalette(Value: boolean);
begin
  inherited SetUsePalette(false);
end;
__WARNON

procedure TEv3Image.Setinverse(AValue: boolean);
begin
  if Finverse=AValue then Exit;
  Finverse:=AValue;
end;



constructor TEv3Image.create();
begin
  inherited create(ev3Width, ev3Height);
end;

end.

